<?php
namespace Ak\App\Config;

use Zend\ConfigAggregator\ArrayProvider;
use Zend\ConfigAggregator\ConfigAggregator;
use Zend\ConfigAggregator\PhpFileProvider;
use Zend\Stdlib\ArrayUtils;

/**
 * Sakuplja konfiguraciju iz raznih izvora.
 *
 * @package Ak\App\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/app-config
 */
final class ConfigFactory
{
    /** @var array */
    private $config;

    /**
     * Konstruktor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = ArrayUtils::merge([
            'autoload_dir' => null,
            'cache_file' => null,
            'development' => false,
        ], $config);
    }

    /**
     * Vraća sakupljenu konfiguraciju.
     *
     * @param array $providers
     * @return array
     */
    public function __invoke(array $providers = []): array
    {
        $development = $this->config['development'] ?? false;

        $providers = ArrayUtils::merge($providers, [
            new ArrayProvider([
                'development' => $development,
                ConfigAggregator::ENABLE_CACHE => !$development,
            ]),
        ]);

        if (is_dir($this->config['autoload_dir'])) {
            $providers[] = new PhpFileProvider(
                rtrim($this->config['autoload_dir'], '/') . '/{{,*.}global,{,*.}local}.php'
            );
        }

        $cacheFile = !$development
            ? ($this->config['cache_file'] ?? null)
            : null;

        if (null !== $cacheFile && !is_dir(dirname($cacheFile))) {
            mkdir(dirname($cacheFile), 0777, true);
        }

        $aggregator = new ConfigAggregator($providers, $cacheFile);
        return $aggregator->getMergedConfig();
    }
}